# -*- coding: utf-8 -*-
# pylama:skip=1
from __future__ import division
from __future__ import print_function
from __future__ import absolute_import
from __future__ import unicode_literals

from .storage import Storage
